from puke import *
import json
import os
import re
from distutils import version
from verlib import NormalizedVersion
from distutils.version import StrictVersion, LooseVersion

AIRSTRIP_ROOT = os.path.dirname(os.path.realpath(__file__))
NORMALIZE_VERSION = re.compile(r'([^\d]*)')

# System-wide yawns path
AIRSTRIP_YAWN_PATH = os.path.join(AIRSTRIP_ROOT, 'airs')

# Project user-defined yawns path
PROJECT_YAWN_PATH = './airs'

EMPTY_GLOBAL = """
      "description": "",
      "keywords": [],
      "author": [],
      "licenses": [],
      "category": "library",
      "homepage": "http://",
      "tools": [],
      "depends": {},
      "strict": true,
      "git": "",
      "versions": {
        "master": {
          "package": "", 
          "resources": {
          },
          "build": [],
          "productions": {
          }
        }
      }
}"""


EMPTY_LOCAL_VERSION = """{
  "versions": {
    "version.name": {
      "package": "", 
      "resources": {
      },
      "build": [],
      "productions": {
      }
    }
  }
}"""

class Air():
  def __init__(self, name):
    self.name = name
    self.hasGlobal = False
    self.yawn = json.loads("""{
      "name": "%s",
      %s""" % (name, EMPTY_GLOBAL))

    systemPath = FileSystem.join(AIRSTRIP_YAWN_PATH, '%s.json' % name)
    if FileSystem.exists(systemPath):
      try:
        self.yawn = json.loads(FileSystem.readfile(systemPath))
        self.hasGlobal = True
      except:
        console.error('The system yawn descriptor for %s is borked!' % name)

    self.hasLocal = False
    self.local = json.loads('{}')
    localPath = FileSystem.join(PROJECT_YAWN_PATH, '%s.json' % name)
    if FileSystem.exists(localPath):
      try:
        self.local = json.loads(FileSystem.readfile(localPath))
        self.hasLocal = True
      except:
        console.error('The yawn descriptor for %s in your project is borked!' % name)


  @staticmethod
  def exists(name):
    systemPath = FileSystem.join(AIRSTRIP_YAWN_PATH, '%s.json' % name)
    localPath = FileSystem.join(PROJECT_YAWN_PATH, '%s.json' % name)
    if not FileSystem.exists(localPath) and not FileSystem.exists(systemPath):
      return False
    return True



  def edit(self, globally = False):
    # Global edition, just go
    if globally:
      p = FileSystem.join(AIRSTRIP_YAWN_PATH, '%s.json' % self.name)
      c = self.yawn
    else:
      p = FileSystem.join(PROJECT_YAWN_PATH, '%s.json' % self.name)
      # No local data yet
      if not self.hasLocal:
        # if no global data either, populate with yawn
        if not self.hasGlobal:
          self.local = json.loads("""{
            "name": "%s",
            %s""" % (self.name, EMPTY_GLOBAL))
        # if has global data, should start empty instead, as a version specialization
        else:
          self.local = json.loads(EMPTY_LOCAL_VERSION)
      c = self.local

    if not FileSystem.exists(p):
      FileSystem.writefile(p, json.dumps(c, indent=4))
    sh('open "%s"' % p)
    self.__init__(self.name)

  def get(self, version, key = False):
    if key == "safename":
      return self.name
    keys = ['name', 'homepage', 'git', 'description', 'author', 'keywords', 'strict', 'licenses', 'category', 'tools', 'depends', 'package', 'resources', 'build', 'productions']

    #, 'versions']
    # if key and not key in keys:
    #   console.error('There is no such thing as %s' % key)

    if self.hasGlobal and (version in self.yawn["versions"]):
      ref = self.yawn['versions'][version]
      if "package.json" in ref and "component.json" in ref:
        for i in ref["component.json"]:
          ref["package.json"][i] = ref["component.json"][i]
      parent = self.yawn
    elif self.hasLocal and (version in self.local["versions"]):
      ref = self.local['versions'][version]
      if "package.json" in ref and "component.json" in ref:
        for i in ref["component.json"]:
          ref["package.json"][i] = ref["component.json"][i]
      parent = self.local
    else:
      console.error('The requested version (%s) does not exist' % version)
      raise Exception("FAIL")

    if not key:
      return ref
    if key in ref:
      return ref[key]
    if "package.json" in ref and key in ref["package.json"]:
      return ref["package.json"][key]

    if not key in parent:
      if "branch" in ref:
        return self.get(ref["branch"], key)
      else:
        console.warn('No such key (%s)' % key)
        return False
    return parent[key]

  def versions(self):
    r = re.compile(r'([^\d]*)')
    versions = {}
    result = []
    dates = {}

    if self.hasGlobal:
      self._parseVersions(self.yawn["versions"], versions)
      # print(self.yawn["versions"])
    if self.hasLocal:
      self._parseVersions(self.local["versions"], versions)
      
    hasMaster = versions.pop('master', False)
    
    sortedVersions = versions.keys()
    sortedVersions.sort(key=LooseVersion)

    for key in sortedVersions:
      result.append(versions[key])

    if hasMaster:
      result.append(hasMaster)

    return result

  def _parseVersions(self, entries, result):
    for version in entries:
      date = False
      content = entries[version]

      if 'date' in content and 'commited' in content['date']:
        date = content['date']['commited']
      
      if not date:
        date = None

      if version == 'master':
        normalized = 'master'
      else:
        normalized = NORMALIZE_VERSION.sub('', version, 1)
        
      result[normalized] = (version, date)

  # def latest(self):
  #   v = self.versions()
  #   for i in v:
  #     v = i.lstrip("v")
  #     better = re.sub(r"([0-9]+)[.]([0-9]+)[.]([0-9]+)(.*)$", r"\1 \2 \3 \4", v).split(" ")
  #     print better
      # try:
      #   print NormalizedVersion(v)
      # except:
      #   print v.split('.')
      #   print "WRONG version"
# http://www.python.org/dev/peps/pep-0386/#normalizedversion
      # version.StrictVersion('1.0.5') < version.StrictVersion('1.0.8')
      # print version.StrictVersion(i)

      # print better
    # if self.hasGlobal:
    #   if key in self.yawn:
    #     ref = self.yawn[key]

    # if self.hasGlobal:
    #   ref = 
    #   return self.yawn[key]
    # if self.hasLocal:
    #   return self.local[key]
    # idx = keys.index(key)
    # types = ['', '', [], [], '', [], {}, '', {}, [], {}]
    # return types[idx]






# -*- coding: utf8 -*-

from puke import *
import os

AIRSTRIP_ROOT = os.path.dirname(os.path.realpath(__file__))

# System-wide yawns path
AIRSTRIP_LICENSES = FileSystem.join(AIRSTRIP_ROOT, 'licenses')

# Template for empty RC
EMPTY_LICENSE = """http://licenseurl

Licensed under the {key} license.

License text, bla."""

class AirLicenses():
  def __init__(self):
    self.licences = {}
    for i in FileList(AIRSTRIP_LICENSES).get():
      d = FileSystem.readfile(i).split('\n\n')
      licenseurl = d.pop(0)
      licensecontent = ('\n\n').join(d)
      self.licences[FileSystem.basename(i).split('.').pop(0).upper()] = {"url": licenseurl, "content": licensecontent}

  def get(self, key):
    key = key.upper()
    if key in self.licences:
      return self.licences[key]
    return False

  def edit(self, key):
    key = key.upper()
    p = FileSystem.join(AIRSTRIP_LICENSES, key)
    # Kind of dirty hack to have it in the git local repo instead of system-wide
    if FileSystem.exists('airstrip/licenses'):
      p = FileSystem.join('airstrip/licenses', key)

    if not key in self.licences:
      FileSystem.writefile(p, EMPTY_LICENSE.replace('{key}', key))
    sh('open "%s"' % p, output = False)

  def list(self):
    return self.licences.keys()



# -*- coding: utf8 -*-

from puke import *
import json

# This file is meant to manipulate the current project "airfile", containing lists of
# what the user has requested

PROJECT_AIRFILE_PATH = './airfile.json'

class AirFile():
  def __init__(self):
    self.project = self._load_()

  def _load_(self):
    ret = json.loads('{}')
    if FileSystem.exists(PROJECT_AIRFILE_PATH):
      try:
        d = json.loads(FileSystem.readfile(PROJECT_AIRFILE_PATH))
        if 'libraries' in d:
          ret = d['libraries']
      except:
        console.error('Your project airfile is horked and has been ignored!')
    return ret

  def _save_(self, data):
    original = json.loads('{}')
    if FileSystem.exists(PROJECT_AIRFILE_PATH):
      try:
        original = json.loads(FileSystem.readfile(PROJECT_AIRFILE_PATH))
      except:
        pass
    original['libraries'] = data

    FileSystem.writefile(PROJECT_AIRFILE_PATH, json.dumps(original, indent=4))


  def require(self, name, version):
    if name in self.project:
      if version in self.project[name]:
        console.error('Library %s in version %s is already required' % (name, version))
        return
      else:
        self.project[name].append(version)
    else:
      self.project[name] = [version]

    self._save_(self.project)


  def remove(self, name, version = False):
    if not name in self.project:
      console.error('That library was not requested in the first place')
      return

    if version and (not version in self.project[name]):
      console.error('That version of the library was not requested in the first place')
      return

    if version:
      self.project[name].remove(version)
      console.info('Library %s version %s has been removed from dependencies' % (name, version))

    if (not version) or (not len(self.project[name])):
      self.project.pop(name, None)
      console.info('Library %s is no longer a dependency of your project' % name)

    self._save_(self.project)

  def list(self):
    for i in self.project:
      print "%s: " % i
      for j in self.project[i]:
        print "    - version: %s" % j




  def isRequired(self, name):
    return name in self.project

  def requiredLibraries(self):
    return self.project



global Tools
from puke import *
from puke import Tools
import json
import re


def fetchsvn(url, dest):
  System.check_package('svn')
  # If directory exist, then update the tree
  if FileSystem.exists(dest):
    console.info('Updating')
    that = 'cd "%s"; svn up' % dest
  else:
    if not FileSystem.exists(FileSystem.dirname(dest)):
      FileSystem.makedir(FileSystem.dirname(dest))
    console.info('Cloning')
    that = 'cd "%s"; svn co %s %s' % (FileSystem.dirname(dest), url, FileSystem.basename(dest))

  # Do the deed
  try:
    std = Std()
    sh(that, std=std, output=False)
    if std.err:
     # and (std.err.find('No stash found.') == -1):
      raise std.err
  except:
    # if puke.FileSystem.exists(dest):
    #   puke.FileSystem.remove(dest)
    console.error('Svn operation failed! %s You need to manually fix or remove the directory.' % std.err)


def fetchone(url, dest, rename):
  remotefilename = url.split('/').pop()
  type = url.split('.').pop().lower()
  # Dirty trick to detect zip where the remote has no extension
  destype = rename.split('.').pop().lower()
  packpath = FileSystem.join(dest, remotefilename)
  if type == 'git':
    packpath = packpath.split('.')
    packpath.pop()
    packpath = '.'.join(packpath)
    console.info('Git repository')
    fetchgit(url, packpath)
    return packpath
  elif type == 'svn' or destype == 'svn':
    console.info('Svn repository %s %s' % (url, packpath))
    fetchsvn(url, packpath)
    return packpath
  else:
    deepcopy(FileList(url), dest + '/')
    if type == 'zip' or type == 'gz' or type == 'bz2' or destype == 'zip':
      try:
        dd = FileSystem.join(dest, remotefilename.replace('.' + type, ''))
        if FileSystem.exists(dd):
          FileSystem.remove(dd)
        FileSystem.makedir(dd)
        unpack(packpath, dd, verbose = False)
        # puke.FileSystem.remove(packpath)
      except Exception as e:
        sh('cd "%s"; 7z x "%s"' % (dd,  FileSystem.abspath(packpath)));
      FileSystem.remove(packpath)
      return FileSystem.join(dest, rename)
    else:
      if remotefilename != rename:
        if not FileSystem.exists(FileSystem.dirname(FileSystem.join(dest, rename))):
          FileSystem.makedir(FileSystem.dirname(FileSystem.join(dest, rename)))
        sh('cd "%s"; mv "%s" "%s"' % (dest, remotefilename, rename))
      packpath = FileSystem.join(dest, rename)

def make(path, command):
  sh('cd "%s"; %s' % (path, command))
  # if type == 'rake':
  #   dorake(path, extra)
  # elif type == 'thor':
  #   dothor(path, extra)
  # elif type == 'make':
  #   domake(path, extra)
  # elif type == 'sh':


# def build(owner, name, versions, tmp, destination):
#   repo = gh.GitHelper(owner, name, tmp)
#   repo.ensure()

  # def __init__(self, owner, name, path):
  # def checkout(self, ref):
  # def getPath(self):



def buildtravis(name, version, ref, travisd, tmp, destination):
  # Dead dirty
  Tools.JS_COMPRESSOR = "%s.js.compress" % sh("which puke", output = False).strip()

def buildone(tmp, name, version, resources, build, productions, destination, strict):
  # Dead dirty
  Tools.JS_COMPRESSOR = "%s.js.compress" % sh("which puke", output = False).strip()

  lastdir = False
  for(localname, url) in resources.items():
    # Do the fetch
    lastdir = fetchone(url, tmp, localname)

  if build:
    if not lastdir:
      console.fail('Build failure not having a directory!')
    for com in build:
      make(lastdir, com)

  if productions:
    if lastdir:
      tmp = lastdir
    for item in productions:
      local = FileSystem.realpath(FileSystem.join(tmp, productions[item]))
      if FileSystem.isfile(local):
        FileSystem.copyfile(local, FileSystem.join(destination, item))
      else:
        deepcopy(FileList(local), FileSystem.join(destination, item))
      if local.split('.').pop().lower() == 'js':
        strict = True
        minify(str(local), re.sub(r"(.*).js$", r"\1-min.js", local), strict = strict)
        FileSystem.copyfile(re.sub(r"(.*).js$", r"\1-min.js", local), FileSystem.join(destination, re.sub(r"(.*).js$", r"\1-min.js", item)))
  else:
    for item in resources:
      local = FileSystem.realpath(FileSystem.join(tmp, item))
      if FileSystem.isfile(local):
        FileSystem.copyfile(local, FileSystem.join(destination, item))
      else:
        deepcopy(FileList(local), FileSystem.join(destination, item))
      if local.split('.').pop().lower() == 'js':
        minify(str(local), re.sub(r"(.*).js$", r"\1-min.js", local), strict = strict)
        FileSystem.copyfile(re.sub(r"(.*).js$", r"\1-min.js", local), FileSystem.join(destination, re.sub(r"(.*).js$", r"\1-min.js", item)))


#         if FileSystem.isfile(f):
#           FileSystem.copyfile(f, d)
#         elif FileSystem.isdir(f):
#           deepcopy(FileList(f), d)



from puke import *
import puke.Cache
import json
import yaml
import os
import base64
import requests
import sys, logging, os, traceback


GITHUB_ROOT = "https://api.github.com"
GITGIT_ROOT = "https://github.com"
GITRAW_ROOT = "https://raw.github.com"

class GitHubInit():
  ########################################
  # HTTP
  ########################################
  def __simpleGet__(self, url):
    # print " [http] simple get %s" % url
    r = requests.get(url)
    return r.text or r.content

  def __cachedGet__(self, u):
    # print " [http] cached get %s" % u
    id = puke.Cache.fetchHttp(u).split('/').pop()
    return puke.Cache.read(id)

  ########################################
  # Github tokens
  ########################################

  def __getToken__(self):
    # print " [github-token] searching for existing auth token"

    d = requests.get("%s/authorizations" % GITHUB_ROOT, auth = self.auth)
    r = json.loads(d.text or d.content)
    for i in r:
      if i["note"] == "airstrip2":
        print " [github-token] found existing auth token %s" % i["token"]
        return i["token"]
    return False

  def __destroyTokens__(self):
    # print " [github-token] destroying auth tokens"

    d = requests.get("%s/authorizations" % GITHUB_ROOT, auth = self.auth)
    r = json.loads(d.text or d.content)
    for i in r:
      if i["note"] == "airstrip2":
        e = requests.delete("%s/authorizations/%s" % (GITHUB_ROOT, i["id"]), auth = self.auth)

  def __createToken__(self):
    # print " [github-token] creating new auth token"

    payload = {"scopes": ["public_repo", "repo"], "note": "airstrip2"}
    headers = {'content-type': 'application/json'}
    d = requests.post("%s/authorizations" % GITHUB_ROOT, data=json.dumps(payload), headers=headers, auth = self.auth)
    r = json.loads(d.text or d.content)
    return r["token"]



  def apiGet(self, fragment):
    if '?' in fragment:
      u = "%s/%s&access_token=%s" % (GITHUB_ROOT, fragment, self.token)
    else:
      u = "%s/%s?access_token=%s" % (GITHUB_ROOT, fragment, self.token)
    r = self.__simpleGet__(u)
    try:
      return json.loads(r)
    except Exception as e:
      console.fail(" [github-connector] Failed json-interpreting url %s with payload %s" % (u, r))

  def apiCacheGet(self, fragment):
    # print " [github-connector] cache fetching %s" % fragment
    u = "%s/%s?access_token=%s" % (GITHUB_ROOT, fragment, self.token)
    r = self.__cachedGet__(u)
    try:
      return json.loads(r)
    except Exception as e:
      console.fail(" [github-connector] Failed json-interpreting cached url %s with payload %s" % (u, r))



  # def buildUrl(self, fragment):
  #   return "%s/%s?access_token=%s" % (GITHUB_ROOT, fragment, self.token)

  def __init__(self):
    # consoleCfg = logging.StreamHandler()
    # consoleCfg.setFormatter(logging.Formatter( ' %(message)s' , '%H:%M:%S'))
    # logging.getLogger().addHandler(consoleCfg)
    # logging.getLogger().setLevel(logging.DEBUG)

    self.uname = 'dmp42'#prompt("Github username")
    self.pwd = '{$githubmaumau$}'#prompt("Github password")

    self.auth = requests.auth.HTTPBasicAuth(self.uname, self.pwd)

    token = self.__getToken__()
    # self.destroyTokens()
    if not token:
      token = self.__createToken__()

    self.token = token

  def search(self, keyword):
    return self.apiGet("legacy/repos/search/%s?sort=stars&order=desc" % (keyword))

  def retrieve(self, owner, repo, dest, name):
    print " [github-connector] working on %s/%s" % (owner, repo)

    # Get refs for a starter
    refs = self.apiGet("repos/%s/%s/git/refs" % (owner, repo))

    print " [github-connector] found %s refs" % len(refs)

    tags = {}
    # Get and init every tag, plus master
    for i in refs:
      tag = i["ref"].split('/').pop()
      if i["ref"].startswith("refs/tags/") or i["ref"].startswith("refs/heads/master"):
        tags[tag] = {"sha": i["object"]["sha"]}
        tags[tag]["tree"] = {}
        tags[tag]["package.json"] = {
          "name": repo,
          "author": owner,
          "version": tag
        }

    print " [github-connector] found %s tags" % len(tags)

    for tag in tags:
      sha = tags[tag]["sha"]
      print " [github-connector] analyzing tag %s (sha %s)" % (tag, sha)

      if tag == "master":
        tree = self.apiGet("repos/%s/%s/git/trees/%s" % (owner, repo, sha))
      else:
        tree = self.apiCacheGet("repos/%s/%s/git/trees/%s" % (owner, repo, sha))

      date = self.apiCacheGet("repos/%s/%s/git/commits/%s" % (owner, repo, sha))
      try:
        tags[tag]["date"] = {
          "authored": date["author"]["date"],
          "commited": date["committer"]["date"]
        }
      except:
        tags[tag]["date"] = {
          "authored": False,
          "commited": False
        }
        print sha
        console.error('Failed fetching a commit!!!')

      for item in tree["tree"]:
        if item["path"].lower() in ['package.json', 'component.json', '.travis.yml']:
          print " [github-connector] actually reading file %s" % item["path"]
          # XXX avoid API call
          item["url"] = "%s/%s/%s/%s/%s" % (GITRAW_ROOT, owner, repo, tag, item["path"].lower())

          if tag == "master":
            d = self.__simpleGet__(item["url"])
          else:
            d = self.__cachedGet__(item["url"])
          try:
            tags[tag][item["path"].lower()] = json.loads(d)
          except:
            try:
              tags[tag][item["path"].lower()] = yaml.load(d)
            except:
              pass
        elif "url" in item:
          tags[tag]["tree"][item["path"]] = item["url"]

    previous = {}
    p = FileSystem.join(dest, '%s.json' % name)
    if FileSystem.exists(p):
      previous = json.loads(FileSystem.readfile(p))

    previous["versions"] = tags
    previous["git"] = "%s/%s/%s" % (GITGIT_ROOT, owner, repo)

    FileSystem.writefile(p, json.dumps(previous, indent=4))



# g = GitHubInit()
# # g.retrieve("documentcloud", "backbone", "airstrip/airs", "backbone")
# # g.retrieve("twitter", "bootstrap", "airstrip/airs", "bootstrap")
# g.retrieve("emberjs", "ember.js", "airstrip/airs", "ember")
# g.retrieve("h5bp", "html5-boilerplate", "airstrip/airs", "h5bp")
# g.retrieve("wycats", "handlebars.js", "airstrip/airs", "handlebars")
# # g.retrieve("jquery", "jquery", "airstrip/airs", "jquery")
# g.retrieve("necolas", "normalize.css", "airstrip/airs", "normalize")
# # g.retrieve("madrobby", "zepto", "airstrip/airs", "zepto")







  # @staticmethod
  # def getblob(url, tmp):
  #   deepcopy(FileList(url), tmp)
  #   content = json.loads(FileSystem.readfile(FileSystem.join(tmp, url.split('/').pop())))
  #   return base64.b64decode(content["content"])

  # @staticmethod
  # def getraw(url, tmp):
  #   deepcopy(FileList(url), tmp)
  #   return FileSystem.readfile(FileSystem.join(tmp, url.split('/').pop()))


# /repos/:owner/:repo/git/trees/:sha

# 4a95dae0378f6e3058f70c51bff03318fb5fc63a






  # config = airc.AirConfig()
  # config.get('temporary')


#!/usr/bin/env puke
# -*- coding: utf8 -*-
from puke.Task import task
global console, FileSystem
from puke import console, FileSystem, sh, deepcopy
global dateutil, datetime
import dateutil.parser
import datetime

global json
import json

# import airfile
global re
import re

global yawn
import air as yawn

global airc
import airconfig as airc

global airf
import airfile as airf

global airb
import airbuild as airb

global gh
import githelper as gh

global ge
import github as ge

global se
import seeder as se

global lic
import airlicenses as lic


@task("Manage available airstrip licenses for projects")
def license(command = False, key = False):
  airlic = lic.AirLicenses()
  if not command or command == 'list':
    print airlic.list()
  elif command == 'edit':
    airlic.edit(key)
  else:
    d = airlic.get(command)
    print "License name: %s" % command.upper()
    print "License url: %s" % d["url"]
    print "License text: %s" % d["content"]



@task("Show current configuration details (airstrip use), or set a configuration flag (airstrip use key value)")
def use(key = False, value = False):
  a = airc.AirConfig()
  if key == False:
    a.list()
  else:
    if not value == False:
      if value.lower() == "true":
        value = True
      # Doesn't f**** work like it should
      elif value.lower() == "false":
        value = False
    a.override(key, value)


@task("Show project required libraries (airstrip require), or add a new library to the list of project dependencies (airstrip require somelibrary), possibly in a specific version (airstrip require somelibrary somversion)")
def require(key = False, version = False):
  a = airf.AirFile()
  if key == False:
    a.list()
  else:
    if not version:
      version = "master"
    # Get a yawn
    if not yawn.Air.exists(key):
      console.fail('No library by that name (%s)!' % key) 
    aero = yawn.Air(key)

    if not version == 'all':
      # Will fail if there is no such version
      aero.get(version, 'name')
      # Otherwise ok!
      a.require(key, version)
    else:
      for i in aero.versions():
        a.require(key, i)


@task("Remove a previously required library from the list of project dependencies (airstrip remove somelibrary), possibly a specific version of it (airstrip remove somelibrary someversion)")
def remove(key, version = False):
  a = airf.AirFile()
  a.remove(key, version)

@task("Edit a library description file (airstrip edit somelibrary). Passing true as second argument edits the descriptor globally.")
def edit(name, globally = False):
  if not globally == False:
    globally = True

  a = yawn.Air(name)
  # Doesn't f**** work like it should
  a.edit(globally)

@task("Show all available libraries (airstrip show), or detailed informations about a specific library (airstrip show somelibrary)")
def show(name = False):
  if not name:
    p = os.path.dirname(os.path.realpath(__file__))
    l = puke.FileList(puke.FileSystem.join(p, 'airs'), filter = "*.json")
    for i in l.get():
      print i.split('/').pop().split('.').pop(0)

    l = puke.FileList('airs', filter = "*.json")
    for i in l.get():
      print i.split('/').pop().split('.').pop(0)

    return
  # XXXX dirty hack
  # name = name.split('/').pop().replace('.json', '')
  if not yawn.Air.exists(name):
    console.fail('No library by that name (%s)!' % name) 
  a = yawn.Air(name)

  nomasterhack = "master"
  try:
    a.get('master', 'name')
  except:
    nomasterhack = a.versions().pop()


  console.info('*********************')
  console.info(a.get(nomasterhack, 'name'))
  console.info('*********************')
  console.info(a.get(nomasterhack, 'description'))
  console.info('*********************')
  # console.info(' - Category: %s' % a.get('master', 'category'))
  console.info(' - Keywords: %s' % a.get(nomasterhack, 'keywords'))
  console.info(' - Homepage: %s' % a.get(nomasterhack, 'homepage'))
  console.info(' - Author: %s' % a.get(nomasterhack, 'author'))
  console.info(' - Licenses: %s' % a.get(nomasterhack, 'licenses'))
  # console.info(' - Required tools to build: %s' % a.get('master', 'tools'))
  console.info('*********************')
  console.info('Available versions:')
  
  versions = a.versions()
  listOfVersions = tuple(x[0] for x in versions)
  maxLength = len(max(listOfVersions, key=len))

  for (i, date) in versions:
    try:
      date = dateutil.parser.parse(date)
      date = datetime.datetime.strftime(date, '%d. %B %Y')
    except:
      date = 'Unkown date'
    console.info(' * %s %s | %s' % ( str(i), " " * (maxLength - len(i)), date))

@task("Initialize new project")
def seed(app = False, mobile = False):
  s = se.Seeder()
  s.project()
  # XXX to be completed
  # executeTask('require', 'jasmine', 'master')
  # executeTask('build')


# @task("List all avalaible libraries")
# def list():
#   p = os.path.dirname(os.path.realpath(__file__))
#   l = puke.FileList(puke.FileSystem.join(p, 'airs'), filter = "*.json")
#   for i in l.get():
#     print i.split('/').pop().split('.').pop(0)

#   l = puke.FileList('airs', filter = "*.json")
#   for i in l.get():
#     print i.split('/').pop().split('.').pop(0)


@task("Search for a given library")
# XXX this is dumb for now
def search(key):

  cachesearch = {}
  result = []
  p = os.path.dirname(os.path.realpath(__file__))
  l = puke.FileList(puke.FileSystem.join(p, 'airs'), filter = "*.json")
  for i in l.get():
    d = puke.FileSystem.readfile(i)
    if key in d:
      result.append(i.split('/').pop().split('.').pop(0))

  result2 = []
  g = ge.GitHubInit()
  d = g.search(key)
  if "repositories" in d:
    p = d["repositories"][0:20]
    for i in p:
      short = "%s/%s" % (i["owner"], i["name"])
      desc = ""
      if i["description"]:
        desc = " %s" % i["description"].replace('\n', ' ')
      result2.append("%s%s" % (short.ljust(50), desc))


  puke.console.warn('Avalaible on github:')
  for i in result2:
    print i

  puke.console.warn('Already installed airs:')
  for i in result:
    print i



@task("Init an air from github")
def init(owner, repo, name = False):
  g = ge.GitHubInit()
  if not name:
    name = repo
  g.retrieve(owner, repo, "airs", name)
  # # g.retrieve("documentcloud", "backbone", "airstrip/airs", "backbone")
  # # g.retrieve("twitter", "bootstrap", "airstrip/airs", "bootstrap")
  # g.retrieve("emberjs", "ember.js", "airstrip/airs", "ember")
  # g.retrieve("h5bp", "html5-boilerplate", "airstrip/airs", "h5bp")
  # g.retrieve("wycats", "handlebars.js", "airstrip/airs", "handlebars")
  # # g.retrieve("jquery", "jquery", "airstrip/airs", "jquery")
  # g.retrieve("necolas", "normalize.css", "airstrip/airs", "normalize")
  # # g.retrieve("madrobby", "zepto", "airstrip/airs", "zepto")

@task("Refresh all airs")
def reinit():
  g = ge.GitHubInit()

  p = os.path.dirname(os.path.realpath(__file__))
  l = puke.FileList(puke.FileSystem.join(p, 'airs'), filter = "*.json")
  for i in l.get():
    name = i.split('/').pop().split('.').pop(0)
    v = json.loads(puke.FileSystem.readfile(i))['git'].split('/')
    repo = v.pop()
    owner = v.pop()
    # print owner, repo, name
    g.retrieve(owner, repo, "airs", name)

  l = puke.FileList('airs', filter = "*.json")
  for i in l.get():
    name = i.split('/').pop().split('.').pop(0)
    v = json.loads(puke.FileSystem.readfile(i))['git'].split('/')
    repo = v.pop()
    owner = v.pop()
    g.retrieve(owner, repo, "airs", name)


@task("Build the list of required libraries, or a specifically required library")
def build(name = False):
  a = airf.AirFile()

  requested = a.requiredLibraries()

  config = airc.AirConfig()
  conftmp = config.get('temporary')
  confdestination = config.get('output')


  if name:
    # Check the library exists and is required
    if not yawn.Air.exists(name):
      console.fail('No library by that name (%s)!' % name) 
    if not a.isRequired(name):
      console.fail('You have not required that library (%s)!' % name) 

    # Build temporary and destination paths for the library

    yawnie = yawn.Air(name)

    nomasterhack = "master"
    try:
      yawnie.get('master', 'name')
    except:
      nomasterhack = yawnie.versions().pop()


    tmp = FileSystem.join(conftmp, yawnie.get(nomasterhack, "safename"))
    destination = FileSystem.join(confdestination, yawnie.get(nomasterhack, "safename"))
    # giti = yawnie.get('master', 'git')
    # Get each version informations json
    # vinfos = {}
    # for version in requested[name]:
    #   vinfos[version] = yawnie.get(version)

    buildit(yawnie, requested[name], tmp, destination)



    return
    # airb.build(owner, name, libs[name], tmp, destination)
    # return

    # for version in requested[name]:
    #   # category = y.get(version, 'category')
    #   destination = FileSystem.join(config.get('output'), name, version)#category, 

    #   data = y.get(version)
    #   if ".travis.yml" in data:
    #     airb.buildtravis(name, version, data[".travis.yml"], tmp, destination)
    #     print data
    #     return
        # print "version %s is OK travis: %s" % (version, data[".travis.yml"])
      # else:
      #   # Try as smart as possible?
      #   if data["tree"]
      #   print "version %s is KO travis" % (version)
                # "node_js": [
                #     0.6
                # ], 
                # "language": "node_js", 
                # "script": "script/test"


      # airb.buildone(tmp, name, version, y.get(version, 'resources'), y.get(version, 'build'), #category, 
      #     y.get(version, 'productions'), destination, y.get(version, 'strict'))
  else:
    for name in requested:

      yawnie = yawn.Air(name)

      nomasterhack = "master"
      try:
        yawnie.get('master', 'name')
      except:
        nomasterhack = yawnie.versions().pop()

      tmp = FileSystem.join(conftmp, yawnie.get(nomasterhack, "safename"))
      destination = FileSystem.join(confdestination, yawnie.get(nomasterhack, "safename"))
      # giti = yawnie.get('master', 'git')
      # Get each version informations json
      # vinfos = {}
      # for version in requested[name]:
      #   vinfos[version] = yawnie.get(version)

      buildit(yawnie, requested[name], tmp, destination)

      # yawnie = yawn.Air(name)
      # tmp = FileSystem.join(config.get('temporary'), yawnie.get("master", "safename"))
      # destination = FileSystem.join(config.get('output'), yawnie.get("master", "safename"))
      # giti = yawnie.get('master', 'git')
      # # Get each version informations json
      # vinfos = {}
      # for version in requested[name]:
      #   vinfos[version] = yawnie.get(version)

      # build(giti, vinfos, tmp, destination)


      # y = yawn.Air(name)

      # for version in requested[name]:
      #   # category = y.get(version, 'category')
      #   tmp = FileSystem.join(config.get('temporary'), name, version)
      #   destination = FileSystem.join(config.get('output'), name, version)

      #   data = y.get(version)
      #   if ".travis.yml" in data:
      #     print "version %s is OK travis: %s" % (version, data[".travis.yml"])

        # print tmp, name, version, y.get(version, 'resources'), y.get(version, 'build'), y.get(version, 'productions'), destination, y.get(version, 'strict')
        # airb.buildone(tmp, name, version, y.get(version, 'resources'), y.get(version, 'build'),
        #     y.get(version, 'productions'), destination, y.get(version, 'strict'))



global buildit
def buildit(yawnie, versions, tmp, dest):

  # XXX horked if a specific version has a specific (different) git url

  nomasterhack = "master"
  try:
    yawnie.get('master', 'name')
  except:
    nomasterhack = yawnie.versions().pop()

  repomanager = gh.GitHelper(yawnie.get(nomasterhack, 'git'), tmp)
  repomanager.ensure()
  p = repomanager.getPath()
  white = ["build", "dist", "test", "tests"]

  for version in versions:
    print "Building version %s" % version

    tree = yawnie.get(version, "tree")

    if version == 'master':
      repomanager.checkout('master')
    else:
      repomanager.checkout(yawnie.get(version, "sha"))


    identified = False
    usenode = False

    trav = yawnie.get(version, ".travis.yml")
    if trav:
      if ("language" in trav) and (trav["language"] == "node_js"):
        usenode = identified = True
      else:
        print "DOESNT KNOW HOW TO BUILD CUSTOM TRAVIS SHIT"

    if yawnie.get(version, "devDependencies"):
      usenode = identified = True

    # Don't build anything but master...
    nob = yawnie.get(version, "nobuild")# or (not version == 'master')
    if nob:
      identified = True

    if not nob:
      usepuke = False
      if "pukefile.py" in tree:
        usepuke = identified = True

      usebundle = False
      if "Gemfile" in tree:
        usebundle = identified = True

      userake = False
      if "Rakefile" in tree:
        userake = identified = True

      usegrunt = False
      if "Gruntfile.js" in tree:
        usegrunt = identified = True
      if "grunt.js" in tree:
        usegrunt = identified = True

      useant = False
      if "build.xml" in tree:
        useant = identified = True

      usemake = False
      if "Makefile" in tree:
        usemake = identified = True

      if usenode:
        puke.sh('cd "%s"; npm install' % p)

      if usebundle:
        puke.sh('cd "%s"; bundle' % p, output = True)

      if usepuke:
        puke.sh('cd "%s"; puke all' % p, output = True)
      elif usegrunt:
        puke.sh('cd "%s"; grunt' % p, output = True)

      elif userake:
        puke.sh('cd "%s"; rake' % p, output = True)

      elif useant:
        puke.sh('cd "%s"; ant' % p, output = True)

      elif usemake:
        puke.sh('cd "%s"; make' % p, output = True)

      elif "build.sh" in tree:
        identified = True
        puke.sh('cd "%s"; ./build.sh' % p, output = True)

      # Yepnope...
      elif "compress" in tree:
        identified = True
        puke.sh('cd "%s"; ./compress' % p, output = True)

      # ES5...
      elif "minify" in tree:
        identified = True
        puke.sh('cd "%s"; ./minify' % p, output = True)

      if usenode:
        scripties = yawnie.get(version, "scripts")
        if scripties:
          for i in scripties:
            if i in white:
              puke.sh('cd "%s"; npm run-script %s' % (p, i))


    if not identified:
      raise "DONT KNOW WHAT TO DO"

    productions = yawnie.get(version, "productions")

    v = version.lstrip("v")
    destination = FileSystem.join(dest, v)
    if productions:
      for item in productions:
        local = productions[item]
        if not local.startswith('http://'):
          local = FileSystem.realpath(FileSystem.join(p, productions[item]))
        if not local.startswith('http://') and not FileSystem.exists(local):
          console.error("Missing production! %s (%s)" % (productions[item], local))
        else:
          if local.startswith('http://'):
            puke.combine(puke.FileList(local), FileSystem.join(destination, item))
          elif not FileSystem.isfile(local):
            puke.deepcopy(puke.FileList(local), FileSystem.join(destination, item))
          else:
            FileSystem.copyfile(local, FileSystem.join(destination, item))



  nos = not yawnie.get(version, "nostrict")
  stuff = puke.FileList(dest, filter = "*.js", exclude = "*-min.js")
  puke.Tools.JS_COMPRESSOR = "%s.js.compress" % puke.sh("which puke", output = False).strip()
  for i in stuff.get():
    mined = re.sub(r"(.*).js$", r"\1-min.js", i)
    if not FileSystem.exists(mined):
      print "Missing minified version %s %s" % (i, mined)
      # XXX strict will blow here
      puke.minify(str(i), mined, strict = nos)


  # XXX too damn dangerous for the benefit - wontfix
  # puke.Tools.CSS_COMPRESSOR = "%s.css.compress" % puke.sh("which puke", output = False).strip()
  # stuff = puke.FileList(dest, filter = "*.css", exclude = "*-min.css")
  # for i in stuff.get():
  #   mined = re.sub(r"(.*).css$", r"\1-min.css", i)
  #   if not FileSystem.exists(mined):
  #     print "Missing minified version %s %s" % (i, mined)
  #     # XXX strict will blow here
  #     try:
  #       puke.minify(str(i), mined, strict = True)
  #     except:
  #       # Bootstrap fail on older versions
  #       print "FAILED COMPRESSION %s" % i






      # if local.split('.').pop().lower() == 'js':
      #   strict = True
      #   minify(str(local), re.sub(r"(.*).js$", r"\1-min.js", local), strict = strict)
      #   FileSystem.copyfile(re.sub(r"(.*).js$", r"\1-min.js", local), FileSystem.join(destination, re.sub(r"(.*).js$", r"\1-min.js", item)))



  # if productions:

  # else:


# @task("Get a specific info about a specific version of a library")
# def get(name, version, key):
#   a = yawn.Air(name)
#   print a.get(version, key)





# @task("Default task")
# def default():
#   print('Victory')
#   pass
  # executeTask("build")
  # executeTask("deploy")


# @task("All")
# def all():
#   executeTask("build")
#   executeTask("mint")
#   executeTask("deploy")
#   executeTask("stats")


# @task("Wash the taupe!")
# def clean():
#   PH.cleaner()

# # Get whatever has been built and exfilter some crappy stuff
# @task("Deploying")
# def deploy():
#   PH.deployer(False)


# @task("Stats report deploy")
# def stats():
#   PH.stater(Yak.build_root)


# @task("Minting")
# def mint():
#   # list = FileList(Yak.build_root, filter = "*bootstrap*.js", exclude = "*-min.js")
#   # for burne in list.get():
#   #   minify(burne, re.sub(r"(.*).js$", r"\1-min.js", burne), strict = False, ecma3 = True)
#   # raise "toto"
#   # These dont survive strict
#   PH.minter(Yak.build_root, filter = "*raphael*.js,*ember*.js,*yahoo*.js,*yepnope*.js,*modernizr*.js,*jasmine*.js", excluding=",*/jax*,*mathjax/fonts*", strict = False)
#   PH.minter(Yak.build_root, excluding = "*raphael*.js,*ember*.js,*yahoo*.js,*yepnope*.js,*modernizr*.js,*jasmine*.js,*/jax*,*mathjax/fonts*", strict = True)

# @task("Deploying the static ressources, including approved third party dependencies")
# def build(buildonly = False):
#   # Crossdomain
#   sed = Sed()
#   sed.add("<\!--.*-->\s*", "")
#   combine("src/crossdomain.xml", Yak.build_root + "/crossdomain.xml", replace = sed)

#   # Robots
#   sed = Sed()
#   # XXX partially fucked-up
#   sed.add("(?:^|\n+)(?:#[^\n]*\n*)+", "")
#   combine("src/robots.txt", Yak.build_root + "/robots.txt", replace = sed)

#   # Deepcopy other stuff
#   sed = Sed()
#   PH.replacer(sed)
#   list = FileList("src/", exclude="*robots.txt,*crossdomain.xml,*index.html")
#   deepcopy(list, Yak.build_root, replace=sed)


#   # Process the remote leaves
#   description = {}

#   # Yak.collection.items()
#   colls = PH.getyanks()
#   # print Yak.collection
#   # for name in Yak.collection:
#   #   print name
#   for name in colls:
#     packinfo = colls[name]
#     # Temporary and build output directories definitions
#     tmpdir = FileSystem.join(Yak.tmp_root, "lib", packinfo["Destination"], name)
#     builddir = FileSystem.join(Yak.build_root, "lib", packinfo["Destination"], name)

#     desclist = []
#     marker = 'lib/%s/' % packinfo["Destination"]
#     for(localname, url) in packinfo["Source"].items():
#       # Do the fetch of 
#       PH.fetchone(url, tmpdir, localname)
#       # Copy files that "exists" to build directory
#       f = FileSystem.join(tmpdir, localname)
#       if FileSystem.exists(f):
#         d = FileSystem.join(builddir, localname)
#         # if not FileSystem.exists(FileSystem.dirname(d)):
#         #   FileSystem.makedir(FileSystem.dirname(d));
#         FileSystem.copyfile(f, d)
#         # Augment desclist with provided localname
#         desclist += [FileSystem.join(marker, name, localname)]

#     if "Build" in packinfo:
#       buildinfo = packinfo["Build"]
#       production = buildinfo["production"]
#       tmpdir = FileSystem.join(tmpdir, buildinfo["dir"])
#       extra = ''
#       if 'args' in buildinfo:
#         extra = buildinfo["args"]
#       if not buildonly or buildonly == name:
#         PH.make(tmpdir, buildinfo["type"], extra)

#       # Copy production to build dir
#       for(local, builded) in production.items():
#         f = FileSystem.join(tmpdir, builded)
#         d = FileSystem.join(builddir, local)
#         desclist += [FileSystem.join(marker, name, local)]
#         if FileSystem.isfile(f):
#           FileSystem.copyfile(f, d)
#         elif FileSystem.isdir(f):
#           deepcopy(FileList(f), d)

#       # ["coin%s" % key for key in ['item1', 'item2']]


#       # map((lambda item: "%s%s" % (name, item)), ['item1', 'item2'])
#       # # Augment description list with build result
#       # bitch = production.keys();

#       # for x in bitch:
#       #   bitch[x] = FileSystem.join(name, bitch[x]);

#       # print bitch
#       # raise "toto"

#       # desclist = desclist + production.keys()

#     description[name] = desclist
#     # description[name] = "%s%s" % (name, marker, ('",\n"%s' % marker).join(desclist)))

#     # miam += """
#     #   %s:
#     #     ["%s%s"]
#     # """ % (name, marker, ('", "%s' % marker).join(desclist))
#   # FileSystem.writefile(FileSystem.join(Yak.build_root, "airstrip.yaml"), yaml.dump(yaml.load('\n'.join(description))))


#     # print json.dumps(description)
#     # raise "toto"

#   shortversion = Yak.package['version'].split('-').pop(0).split('.')
#   shortversion = shortversion[0] + "." + shortversion[1]
#   PH.describe(shortversion, "airstrip", description)
#   # Write description file
#   # FileSystem.writefile(FileSystem.join(Yak.build_root, "airstrip.json"), '{%s}' % ',\n'.join(description))

#   # Build-up the description file
#   file = "src/index.html"
#   sed.add("{PUKE-LIST}", json.dumps(description, indent=4))
#   deepcopy(file, Yak.build_root, replace=sed)





#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__ = 'airstrip'
__version__ = '2.0.2'
__build__ = 0x001400
__author__ = 'WebItUp'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 WebItUp'

import sys, logging, os, traceback
import pkg_resources
from optparse import OptionParser
import puke

try:
    sys.path.insert(1, os.getcwd())
except:
    pass

def run():
    """ Main routine which should be called on startup """

    #
    # Parse options
    #

    parser = OptionParser()
    parser.add_option("-v", "--version", action="store_true", dest="version", help="get version")
    # parser.add_option("-h", "--help", action="store_true", dest="help", help="get help")

    (options, args) = parser.parse_args()
    consoleCfg = logging.StreamHandler()
    consoleCfg.setFormatter(logging.Formatter( ' %(message)s' , '%H:%M:%S'))
    logging.getLogger().addHandler(consoleCfg)
    logging.getLogger().setLevel(logging.DEBUG)
    #
    # Execute tasks
    #


    if options.version:
        print('AirStrip %s' % __version__)
        sys.exit(0)


    # if options.help:
    #     puke.printTasks()
    #     sys.exit(0)

   
    if os.path.isfile('.pukeignore'):
        try:
            f = open('.pukeignore', 'r')

            for line in f:
                puke.FileList.addGlobalExclude(line.strip())
        except Exception as e:
            print('Puke ignore error : %s' % e)

    #
    # Find and execute build script
    #
    
    pukefiles = ["pukefile", "pukeFile", "pukefile", "pukefile.py", "pukeFile.py", "pukefile.py"]
    
    working_dir = os.path.dirname(os.path.realpath(__file__))
    script = None
    for name in pukefiles:
      if os.path.isfile(os.path.join(working_dir, name)):
        script = os.path.join(working_dir, name)

    
    retval = execfile(script)

    
    
    try:
        args = args.strip()
    except:
        pass

    if not args:
        if puke.hasDefault():
            puke.executeTask('default')
        else:
            logging.error("No tasks to execute. Please choose from: ")
            puke.printTasks()
            sys.exit(1)

    else:
        name = args.pop(0)
        puke.executeTask(name.strip(), *args)
                


def gettraceback(level = 0):
    trace = ""
    exception = ""
    exc_list = traceback.format_exception_only (sys.exc_type, sys.exc_value)

    reverse = -1 - level
    for entry in exc_list:
        exception += entry
    tb_list = traceback.format_tb(sys.exc_info()[2])
    for entry in tb_list[reverse]:
        trace += entry  
    return trace  

def main():
    try:
        run()
    
    except Exception as error:

        print("ERROR %s \n %s \n" % (error, gettraceback()))

        sys.exit(1)
        
    except KeyboardInterrupt:
        print("Build interrupted!\n")
        sys.exit(2)
        
    sys.exit(0)


# -*- coding: utf8 -*-

from puke import *
import json
import os
import datetime
import re

import airrc
import airlicenses

# import airrc

AIRSTRIP_ROOT = os.path.dirname(os.path.realpath(__file__))

class Seeder():
  def __init__(self):
    pass

  def project(self):
    self.rc = airrc.AirRC()


    console.info("""This will seed a project in the current working directory.
You can leave any of the following informations blank, or later edit the generated files to fit your mileage.""")

    # Existing git repo
    std = Std()
    gitdata = sh('git remote -v', std = std)
    if not std.err:
      gitdata = gitdata.split('\n')
      gitdata.pop()
      gitdata = gitdata.pop().split(' ')
      gitdata.pop()
      gitdata = gitdata.pop().split('\t')
      gitdata = gitdata.pop().split('/')

      clean = re.sub('[.]git$', '', gitdata.pop())
      gitdata = {'repo': clean, 'owner': gitdata.pop().split(':').pop()}
    else:
      console.error('Current directory is not a git repository. Git data will be missing.')
      gitdata = False

    # User-provided informations
    defaultname = FileSystem.basename(FileSystem.realpath('.'))
    name = prompt("Pick a fancy project name (default %s)" % defaultname, defaultname).strip()
    description = prompt("Short description for your project").strip()
    k = prompt("Keywords for your project (comma separated)").strip().split(',')
    keywords = []
    for i in k:
      i = i.lower().strip()
      if i:
        keywords.append(i)
    keywords = json.dumps(keywords)


    # Default suggestions?
    yes = prompt("""Other informations can be automatically populated from your defaults (.airstriprc).
Would you like that (default, "y"), or would you prefer to review them one by one ("N")?""", "y")

    if not self.rc.get('you')['name']:
      self.rc.get('you')['name'] = Env.get("PUKE_LOGIN", System.LOGIN)

    you = self.rc.get('you').copy()
    company = self.rc.get('company').copy()
    homepage = company['url']
    license = self.rc.get('license')
    ln = self.rc.get('ln')
    git = self.rc.get('git')

    airl = airlicenses.AirLicenses()
    if yes.lower() == 'y':
      if not gitdata:
        gitdata = {'repo': git, 'owner': defaultname}
    else:
      company["name"] = prompt("The author name of the project and copyright owner (default: %s)" % company["name"], company["name"]).strip()
      company["mail"] = prompt("Author email (default: %s)" % company["mail"], company["mail"]).strip()
      company["url"] = prompt("Author url (default: %s)" % company["url"], company["url"]).strip()

      you["name"] = prompt("The principal package maintainer name of the project (default: %s)" % you["name"], you["name"]).strip()
      you["mail"] = prompt("Maintainer email (default: %s)" % you["mail"], you["mail"]).strip()
      you["url"] = prompt("Maintainer url (default: %s)" % you["url"], you["url"]).strip()

      keys = airl.list()
      seg = prompt("License for your project (choose from: %s, default: %s)" % (keys, license or 'MIT'), license or 'MIT').strip().upper()
      if not seg in keys:
        seg = license or 'MIT'
        console.error('Unknown license, fallbacking to %s' % seg)
      license = seg


      seg = prompt("License for your project (choose from: %s, default: %s)" % (keys, license or 'MIT'), license or 'MIT').strip().upper()
      ln = prompt("Language for your project (default: %s)" % ln, ln).strip()

      homepage = prompt("Homepage for your project? (default: %s)" % homepage, homepage).strip()

      if not gitdata:
        gitdata = prompt("Github owner/repositoryname (default: %s/%s)" % (git, defaultname), "%s/%s" % (git, defaultname)).strip()
        if gitdata:
          gitdata = gitdata.split('/')
          gitdata = {'repo': gitdata.pop(), 'owner': gitdata.pop()}
        else:
          gitdata = False

    # Preparing replacement patterns from provided information
    s = Sed()
    s.add('{{miniboot.name}}', name)
    s.add('{{miniboot.ln}}', ln)
    s.add('{{miniboot.description}}', description)
    s.add('{{miniboot.keywords}}', keywords)
    s.add('{{miniboot.homepage}}', homepage)

    s.add('{{miniboot.you.name}}', you['name'])
    s.add('{{miniboot.you.mail}}', you['mail'])
    s.add('{{miniboot.you.web}}', you['url'])
    s.add('{{miniboot.company.name}}', company['name'])
    s.add('{{miniboot.company.mail}}', company['mail'])
    s.add('{{miniboot.company.web}}', company['url'])

    now = datetime.datetime.now()
    s.add('{{miniboot.year}}', str(now.year))

    s.add('{{miniboot.path}}', FileSystem.realpath('.'))

    track = ''
    rep = ''
    if gitdata:
      track = 'https://github.com/%s/%s/issues' % (gitdata["owner"], gitdata["repo"])
      rep = 'https://github.com/%s/%s.git' % (gitdata["owner"], gitdata["repo"])

    s.add('{{miniboot.tracker}}', track)
    s.add('{{miniboot.repository}}', rep)

    lurl = airl.get(license)['url']
    lname = license

    s.add('{{miniboot.license.name}}', lname)
    s.add('{{miniboot.license.url}}', lurl)

    items = FileList(FileSystem.join(AIRSTRIP_ROOT, 'boilerplates'))
    deepcopy(items, '.', replace = s)

    d = FileSystem.readfile('LICENSE.md')
    FileSystem.writefile('LICENSE.md', '%s\n\n%s' % (d, airl.get(license)['content']))

    FileSystem.writefile('package-%s-%s.json' % (System.LOGIN, System.OS.lower()), '{}')
    deepcopy(FileList(FileSystem.join(AIRSTRIP_ROOT, 'boilerplates', 'src')), './src', replace = s)





#!/usr/bin/env puke
# -*- coding: utf8 -*-

global help
from helpers import Helpers as help
import re
import json

@task("Default task")
def default():
  executeTask("build")
  executeTask("deploy")

@task("All")
def all():
  # Cache.clean()
  executeTask("lint")
  executeTask("hint")
  executeTask("build")
  executeTask("tests")
  executeTask("mint")
  executeTask("deploy")
  executeTask("doc")
  executeTask("stats")


@task("Wash the taupe!")
def clean():
  Cache.clean()
  help.cleaner()

@task("jsDocking")
def doc():
  list = FileList(Yak.paths['build'], filter = "*.js")
  # jsdoc3(list, Yak.doc_root + "/jsdoc3.json")
  d = FileSystem.abspath(Yak.paths["doc"])
  jsdoc3(list, "%s/gristaupe.json" % d)
  jsdoc3(list, "%s/html" % d, template = "templates/default")

@task("Lint")
def lint():
  help.linter("src")

@task("Hint")
def hint():
  help.hinter("src")

@task("Flint")
def flint():
  help.flinter("src")

@task("Minting")
def mint():
  help.minter(Yak.paths['build'], strict = True)
  # Some dirty code might not pass strict
  # help.minter(Yak.paths['build'], strict = False)

@task("Stats report deploy")
def stats():
  help.stater(Yak.paths['build'])

@task("Tests building")
def tests():
  # ============================
  # Build tests
  # ============================
  sed = Sed()
  help.replacer(sed)

  list = FileList(Yak.paths['tests'], filter="*.js,*.html,*.css")
  deepcopy(list, Yak.paths['build'] + '/tests', replace=sed)


@task("Build package")
def build():
  # ============================
  # Very basic build
  # ============================

  sed = Sed()
  help.replacer(sed)
  deepcopy(FileList("src", exclude = "*tests*"), Yak.paths['build'], replace = sed)


@task("Deploy package")
def deploy():
  # Libraries usually have a versioned path (True)
  # help.deployer(True)
  # Sites or apps dont
  help.deployer(Yak.paths['build'], False)
  # In case you wanna deploy dependencies as well
  help.deployer('dependencies', False, 'dependencies')



# -*- coding: utf8 -*-

from puke import *
import json
import airlicenses as al

AIRSTRIP_RC_PATH = '~/.airstriprc'

API = '2'

# Template for empty RC
EMPTY_RC = json.loads("""{
  "version": "",
  "company": {
    "name": "",
    "url": "",
    "mail": ""
  },
  "git": "",
  "ln": "en-us",
  "you": {
    "name": "",
    "url": "",
    "mail": ""
  },
  "license": "MIT"
}""")

class AirRC():
  def __init__(self):
    if not FileSystem.exists(AIRSTRIP_RC_PATH):
      FileSystem.writefile(AIRSTRIP_RC_PATH, json.dumps(EMPTY_RC, indent = 2))

    try:
      self.rc = json.loads(FileSystem.readfile(AIRSTRIP_RC_PATH))
    except:
      console.fail('Your airstrip rc file (%s) is horked! Please rm or fix it' % AIRSTRIP_RC_PATH)

    if not self.rc['version'] == API:
      self.__ask__()

  def __ask__(self):
    defaults = self.rc.copy()
    for i in EMPTY_RC:
      if not i in defaults:
        defaults[i] = EMPTY_RC[i]

    console.warn("""You don't seem to have documented your default informations, 
or airstrip has an upgraded version that requires new infos.""")
    console.info("""These infos are stored only in the file %s, which you can edit manually, 
are entirely optional, and used only by the airstrip "seed" command to populate package.json 
and other projects boilerplates.""" % AIRSTRIP_RC_PATH)

    console.info('First, provide informations about your company (if any - used generally for the author fields and copyright owner informations.)')
    self.rc['company']['name'] = prompt('Your company name (currently: %s)' % defaults['company']['name'], defaults['company']['name'])
    self.rc['company']['mail'] = prompt('Your company mail (currently: %s)' % defaults['company']['mail'], defaults['company']['mail'])
    self.rc['company']['url'] = prompt('Your company website / twitter (currently: %s)' % defaults['company']['url'], defaults['company']['url'])

    console.info('Now, about you - this will be used for the contributors/maintainers fields.')
    self.rc['you']['name'] = prompt('Your name (currently: %s)' % defaults['you']['name'], defaults['you']['name'])
    self.rc['you']['mail'] = prompt('Your mail (currently: %s)' % defaults['you']['mail'], defaults['you']['mail'])
    self.rc['you']['url'] = prompt('Your website / twitter (currently: %s)' % defaults['you']['url'], defaults['you']['url'])

    keys = al.AirLicenses().list()
    self.rc['license'] = prompt('Default license for new projects (among %s)? (currently: %s)' % (keys, defaults['license']), defaults['license'])

    self.rc['git'] = prompt('Default git owner to use for new projects? (currently: %s)' % defaults['git'], defaults['git'])
    self.rc['ln'] = prompt('Default language for projects? (currently: %s)' % defaults['ln'], defaults['ln'])

    self.set('version', API)

  def get(self, key):
    if key in self.rc:
      return self.rc[key]
    return None

  def set(self, key, value):
    if key:
      self.rc[key] = value
    FileSystem.writefile(AIRSTRIP_RC_PATH, json.dumps(self.rc, indent = 2))



# -*- coding: utf8 -*-

from puke import *
import re
import json
import os

# ==================================================================
# Global helpers for puke
# ==================================================================

# ------------------------------------------------------------------
# Common yak soup
# ------------------------------------------------------------------

# Mint every file in the provided path avoiding xxx files, tests, and already mint files themselves (usually the build root)

class Helpers:
  @staticmethod
  def enforceunix():
    # Puke at large is just untested on windows - and specifically these helpers
    if System.OS != System.MACOS and System.OS != System.LINUX:
      console.fail('Your platform is not supported')

  @staticmethod
  def loadyanks():
    # Yank the base config file
    working_dir = os.path.dirname(os.path.realpath(__file__))

    r = Require("package.json")
    try:
      r.merge(os.path.join(working_dir, 'config.yaml'))
    except:
      pass

    # Try to get separate user specific file, either as json or yaml
    usercpath = 'package-%s-%s' % (Env.get("PUKE_LOGIN", System.LOGIN), Env.get("PUKE_OS", System.OS).lower())
    try:
      r.merge(usercpath + ".json")
    except:
      try:
        r.merge(usercpath + ".yaml")
      except:
        if System.LOGIN == 'root':
          console.fail('You are trying to puke as root without any PUKE_LOGIN specified that matches a known configuration. This is verbotten!')
        pass

    # Map to older format for lazyness reasons :)
    clean = re.sub('[.]git$', '', r['repositories'][0]["url"])

    r['yak'] = {
      "package": {
        "name": r["name"],
        "version": r["version"],
        "homepage": r["homepage"]
      },
      "rights": {
        "license": '<a href="%s">%s</a>' % (r["licenses"][0]["url"], r["licenses"][0]["type"]),
        "copyright": 'All rights reserved <a href="http://www.webitup.fr">copyright %s</a>' % r["author"],
        "author": r["author"]
      },
      "git": {
        "root": '%s/blob/master/src' % clean
      },
      "paths": r["directories"],
      "config": r["config"]
    }

    r.yak('yak')
    # Git in the yanks
    try:
      branch = sh("cd .; git branch | grep '*'", output=False).strip('*').strip()
      if branch == '(no branch)':
        branch = sh("cd .; git describe --tags", output=False).strip()
      commitnb = sh("cd .; git log --pretty=format:%s | wc -l" % '%h', output=False).strip()
      commithash = sh("cd .; git log | head -n 1 | cut -f2 -d' '", output=False).strip()
      Yak.git['root'] = Yak.git['root'].replace('/master/', '/' + branch + '/')
      Yak.git['revision'] = '#' + commitnb + '-' + commithash
    except:
      Yak.git['revision'] = '#no-git-information'
      console.error("FAILED fetching git information - locations won't be accurate")

    for (key, path) in Yak.paths.items():
      FileSystem.makedir(path)



# def __preparepaths():
#   # Aggregate package name and version to the "root" path, if not the default
#   if Yak.root != './':
#     Yak.root = FileSystem.join(Yak.root, Yak.package['name'], Yak.package['version'])

#   # Aggregate all inner paths against the declared ROOT, and build-up all the corresponding top level Yak variables
#   for (key, path) in Yak.paths.items():
#     # Build-up global key only if not overriden
#     if not (key + '_root') in Yak:
#       Yak.set(key + '_root', FileSystem.join(Yak.root, path))
#     FileSystem.makedir(Yak.get(key + '_root'))


# def __prepareconfig():
#   Yak.istrunk = Yak.settings['variant'] == 'bleed'

  # XXX still crap
  @staticmethod
  def minter(path, filter = '', excluding = '', strict = True):
    if excluding:
      excluding = ',%s' % excluding

    if not filter:
      filtre = '*.js'
      list = FileList(path, filter = filtre, exclude = "*-min.js,%s" % excluding)
      for burne in list.get():
        print burne
        print re.sub(r"(.*).js$", r"\1-min.js", burne)
        minify(str(burne), re.sub(r"(.*).js$", r"\1-min.js", burne), strict = strict)
      # filtre = '*.css'
      # list = FileList(path, filter = filtre, exclude = "*-min.css,%s" % excluding)
      # for burne in list.get():
      #   print burne
      #   print re.sub(r"(.*).js$", r"\1-min.js", burne)
      #   minify(str(burne), re.sub(r"(.*).css$", r"\1-min.css", burne))
    else:
      filtre = filter
      list = FileList(path, filter = filtre, exclude = "*-min.js,%s" % excluding)
      for burne in list.get():
        print burne
        print re.sub(r"(.*).js$", r"\1-min.js", burne)
        minify(str(burne), re.sub(r"(.*).js$", r"\1-min.js", burne), strict = strict)

  # Lint every file (usually src)
  @staticmethod
  def linter(path, excluding = '', relax=False):
    if excluding:
      excluding = ',%s' % excluding
    list = FileList(path, filter = "*.js", exclude = "*-min.js,%s" % excluding)
    jslint(list, relax=relax)

  @staticmethod
  def hinter(path, excluding = '', relax=False):
    System.check_package('node')
    System.check_package('npm')
    System.check_package('jshint')
    if excluding:
      excluding = ',%s' % excluding
    list = FileList(path, filter = "*.js", exclude = "*-min.js,%s" % excluding)
    res = '"' + '" "'.join(list.get()) + '"'
    ret = sh('jshint %s' % res, output = False)
    if ret:
      console.fail(ret)
    else:
      console.info("You passed the dreaded hinter!")

  #  npm install -g jshint


  # Flint every file (usually src)
  @staticmethod
  def flinter(path, excluding = '', relax=False):
    if excluding:
      excluding = ',%s' % excluding
    list = FileList(path, filter = "*.js", exclude = "*-min.js,%s" % excluding)
    jslint(list, relax=relax, fix=True)

  # Stat every file (usually build)
  @staticmethod
  def stater(path, excluding = ''):
    if excluding:
      excluding = ',%s' % excluding
    list = FileList(path, filter = "*.js", exclude = "*-min.js,%s" % excluding)
    stats(list, title = "Javascript")
    list = FileList(path, filter = "*-min.js", exclude = "%s" % excluding)
    stats(list, title = "Minified javascript")
    list = FileList(path, filter = "*.css", exclude = "*-min.css,%s" % excluding)
    stats(list, title = "Css")
    list = FileList(path, filter = "*-min.css", exclude = "%s" % excluding)
    stats(list, title = "Minified css")
    list = FileList(path, filter = "*.html,*.xml,*.txt", exclude = "%s" % excluding)
    stats(list, title = "(ht|x)ml + txt")
    list = FileList(path, exclude = "*.html,*.xml,*.txt,*.js,*.css,%s" % excluding)
    stats(list, title = "Other")


  # # Cleans every "ROOT" folder cautiously
  @staticmethod
  def cleaner():
    for (key, path) in Yak.paths.items():
      if not key == "src" and not key == "tests":
        resp = prompt('Delete %s? y/[N]' % path, 'N')
        if resp == 'y':
          try:
            FileSystem.remove(path)
            console.info('Deleted %s' % path)
          except:
            console.error('Failed removing %s' % path)

  @staticmethod
  def replacer(s):
    for (key, value) in Yak.package.items():
      s.add('{PUKE-PACKAGE-%s}' % key.replace('_', '-').upper(), str(value))
    for (key, value) in Yak.rights.items():
      s.add('{PUKE-RIGHTS-%s}' % key.replace('_', '-').upper(), str(value))
    for (key, value) in Yak.git.items():
      s.add('{PUKE-GIT-%s}' % key.replace('_', '-').upper(), str(value))
    for (key, value) in Yak.paths.items():
      s.add('{PUKE-%s-ROOT}' % key.replace('_', '-').upper(), str(value))
    for (key, value) in Yak.config.items():
      s.add('{PUKE-CONFIG-%s}' % key.replace('_', '-').upper(), str(value))
    return s


  @staticmethod
  def deployer(src, withversion = False, destination = False):
    list = FileList(src)
    if withversion and Yak.paths['dist'] != 'dist':
      v = Yak.package['version'].split('-').pop(0).split('.')
      d = FileSystem.join(Yak.paths['dist'], Yak.package['name'], v[0] + "." + v[1])
      if destination:
        d = FileSystem.join(d, destination)
      deepcopy(list, d)
    else:
      d = Yak.paths['dist']
      if destination:
        d = FileSystem.join(d, destination)
      deepcopy(list, d)


# def __preparepaths():
#   # Aggregate package name and version to the "root" path, if not the default
#   if Yak.root != './':
#     Yak.root = FileSystem.join(Yak.root, Yak.package['name'], Yak.package['version'])

#   # Aggregate all inner paths against the declared ROOT, and build-up all the corresponding top level Yak variables
#   for (key, path) in Yak.paths.items():
#     # Build-up global key only if not overriden
#     if not (key + '_root') in Yak:
#       Yak.set(key + '_root', FileSystem.join(Yak.root, path))
#     FileSystem.makedir(Yak.get(key + '_root'))


# def __prepareconfig():
#   Yak.istrunk = Yak.settings['variant'] == 'bleed'


Helpers.enforceunix()
Helpers.loadyanks()

# ------------------------------------------------------------------
# Top-level helpers
# ------------------------------------------------------------------




# # Adds a {PUKE-*-*} pattern for every package, link, or path entry in the Yak






# def describe(shortversion, name, description):
#   yamu = FileSystem.join(Yak.deploy_root, "%s.json" % name)
#   if FileSystem.exists(yamu):
#     mama = json.loads(FileSystem.readfile(yamu))
#     mama[shortversion] = description
#   else:
#     mama = {shortversion: description}

#   # Straight to service root instead - kind of hackish...
#   FileSystem.writefile(yamu, json.dumps(mama, indent=4))

# # ------------------------------------------------------------------
# # Dedicated airstrip helpers
# # ------------------------------------------------------------------

# def getyanks():
#   # Airstrip yank in additional description files
#   l = FileList('yanks', filter = '*.yaml', exclude = '*xxx*');
#   yanks = {}
#   for i in l.get():
#     a = Load(i)
#     yanks = Utils.deepmerge(yanks, a.content['yanks'])

#   Yak.collection = yanks
#   return yanks


# # Bulding / fetching helpers
# def donode(path, extra):
#   System.check_package('node')
#   sh('cd "%s"; node %s' % (path, extra))

# def dorake(path, extra = ''):
#   System.check_package('rvm')
#   System.check_package('npm')
#   System.check_package('bundle')
#   System.check_package('rake')
#   # XXX handlebars requires node as well :/
#   System.check_package('node')
#   sh('cd "%s"; bundle; rake %s' % (path, extra))

# def dothor(path, extra = ''):
#   System.check_package('rvm')
#   System.check_package('bundle')
#   # System.check_package('tilt')
#   # System.check_package('compass')
#   sh('cd "%s"; bundle; thor %s' % (path, extra))

# def domake(path, extra = ''):
#   sh('cd "%s"; make %s' % (path, extra))






# def make(path, type, extra = ''):
#   if type == 'rake':
#     dorake(path, extra)
#   elif type == 'thor':
#     dothor(path, extra)
#   elif type == 'make':
#     domake(path, extra)
#   elif type == 'sh':
#     sh('cd "%s"; %s' % (path, extra))

  # for (k, ipath) in production.items():
  #   FileSystem.copyfile(FileSystem.join(path, ipath), FileSystem.join(destination, k))

  # else:
  #   sh('cd "%s"; cp -R %s %s' % (path, latest, destination), output = True)
    #   sh("cd " + Yak.TMP_ROOT + "/lib/" + burne["Destination"] + "; cp -R " + burne["Latest"] + " " +  k + "; rm " + burne["Latest"])


      # localtmp = puke.FileSystem.join(tmp, url.split('/').pop())
      # if puke.FileSystem.checksum(localtmp) != self.__checksum:
      #   console.fail("PANIC! Archive doesn't pan out. You may puke -c if in doubt, and anyhow double check integrity. %s vs. %s" % (puke.FileSystem.checksum(localtmp), self.__checksum))

      # if type == 'dmg':
      #   console.info('Processing dmg')
      #   self.__dodmg(localtmp, self.local, pwd)
      # elif type == 'pkg':
      #   console.info('Processing pkg')
      #   self.__dopkg(localtmp)
      # else:
      #   console.info('Processing archive file')
      #   self.__dounpack(localtmp, puke.FileSystem.dirname(pwd))








# -*- coding: utf8 -*-

from puke import *
import json
import os

# That file is meant to manipulate the airstrip configuration in the scope of the current project
# That project configuration by-default use the airstrip global configuration
# Possibly overriden by specific elements in the airconfig file in cwd.
AIRSTRIP_ROOT = os.path.dirname(os.path.realpath(__file__))
# Should point to the globally installed airstrip configuration file
AIRSTRIP_CONFIG_PATH = os.path.join(AIRSTRIP_ROOT, 'global.json')
# The project in the current directory airconfig file, if any
PROJECT_CONFIG_PATH = './airfile.json'


class AirConfig():
  def __init__(self):
    self.general = json.loads(FileSystem.readfile(AIRSTRIP_CONFIG_PATH))
    self.project = self._load_()

  def _load_(self):
    ret = json.loads('{}')
    if FileSystem.exists(PROJECT_CONFIG_PATH):
      try:
        d = json.loads(FileSystem.readfile(PROJECT_CONFIG_PATH))
        if 'config' in d:
          ret = d['config']
      except:
        console.error('Your project file configuration is horked and has been ignored!')
    return ret

  def _save_(self, data):
    original = json.loads('{}')
    if FileSystem.exists(PROJECT_CONFIG_PATH):
      try:
        original = json.loads(FileSystem.readfile(PROJECT_CONFIG_PATH))
      except:
        pass
    original['config'] = data

    FileSystem.writefile(PROJECT_CONFIG_PATH, json.dumps(original, indent=4))


  def list(self):
    for i in self.general:
      value = self.general[i]['default']
      if i in self.project:
        value = '%s (default: %s)' % (self.project[i], self.general[i]['default'])
      print '%s: %s [%s]' % (i, value, self.general[i]['info'])


  def get(self, key):
    if not key in self.general:
      console.error('No such configuration flag (%s)' % key);
      return
    if key in self.project:
      return self.project[key]
    return self.general[key]['default']

  def override(self, key, value):
    if not key in self.general:
      console.error('You are tryin to set a configuration switch that does not exist (%s)' % key);
      return
    if key in self.project:
      # Same value, ignore
      if value == self.project[key]:
        console.error('Ignoring unchanged property %s (value is already %s)' % (key, value))
        return
      # Default value, remove from self.project settings
      if self.general[key]['default'] == value:
        self.project.pop(key, None)
      # Otherwise change self.project key override
      else:
        self.project[key] = value
    elif self.general[key]['default'] == value:
      console.error('Ignoring unchanged property %s (default is already %s)' % (key, value))
      return
    else:
      self.project[key] = value

    self._save_(self.project)

    console.info('Configuration switch "%s" has been set to "%s"' % (key, value))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys

from setuptools import setup, find_packages
import airstrip

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

os.environ['PYTHONDONTWRITEBYTECODE'] = '1'

requires = [
    'puke',
    'verlib'
]

setup(
    name='AirStrip',
    version=airstrip.__version__,
    description='Third-party js dependencies manager',
    long_description=open('README.md').read(),
    author='WebItUp',
    author_email='tech@webitup.fr',
    url='http://airstrip.jsboot.com/',
    packages=find_packages(),
    scripts=[
        'airstrip/bin/airstrip'
    ],
    package_dir={'airstrip': 'airstrip'},
    package_data = {
        # If any package contains *.txt files, include them:
        '': ['*.json']
    },
    include_package_data = True,

    install_requires=requires,
    license=open('LICENSE.md').read()
)



# -*- coding: utf8 -*-

from puke import *
import re

class GitHelper():
  def __init__(self, remote, path):
    # Require git on the system to have it
    System.check_package('git')
    clean = re.sub('[.]git$', '', remote.split('/').pop())
    self.local = FileSystem.join(path, clean)
    self.remote = remote
    self.debug = True

  def __wrap__(self, path, command):
    if FileSystem.realpath('.') == FileSystem.realpath(path):
      console.fail('SOMETHING TERRIBLE WAS ABOUT TO HAPPEN %s' % path)
    std = Std()
    sh('cd "%s";' % path, std = std)
    if std.err:
      console.fail('SOMETHING VERY BAD HAPPENED')
    sh('cd "%s"; git %s' % (path, command), std = std, output = self.debug)
    if std.err and self.debug:
      console.error(std.err)


  def __clean__(self):
    self.__wrap__(self.local, 'reset --hard HEAD; git clean -f -d; git checkout master') # checkout .; git checkout master

  def __rebase__(self):
    self.__wrap__(self.local, 'pull --rebase')

  def __clone__(self):
    self.__wrap__(FileSystem.dirname(self.local), 'clone %s' % self.remote)

  def ensure(self):
    base = FileSystem.dirname(self.local)
    if not FileSystem.exists(base):
      FileSystem.makedir(base)
    if not FileSystem.exists(self.local):
      self.__clone__()
    else:
      self.__clean__()
      self.__rebase__()

  def checkout(self, ref):
    self.__wrap__(self.local, 'reset --hard HEAD; git clean -f -d; git checkout %s; git reset --hard %s; git clean -f -d;' % (ref, ref))

  def getPath(self):
    return self.local

